#include <SPI.h>
#include "KANJIROM.h"

void KANJIROM::init() {
  /*SPI通信の設定*/
  pinMode (SS, OUTPUT);
  SPI.begin ();
  SPI.setBitOrder(MSBFIRST);
  SPI.setDataMode(3);
  SPI.setClockDivider(SPI_CLOCK_DIV16);
}

// 漢字ROM READ : SJISコード
void KANJIROM::readSJISFont(uint16_t sjisCode) {
  // ----- SJIS -> JISx0208変換
  uint8_t c1 = (sjisCode & 0xFF00) >> 8;
  uint8_t c2 = sjisCode & 0xFF;
  if (c1 >= 0xe0) {
    c1 = c1 - 0x40;
  }
  if (c2 >= 0x80) {
    c2 = c2 - 1;
  }
  if (c2 >= 0x9e) {
    c1 = (c1 - 0x70) * 2;
    c2 = c2 - 0x7d;
  } else {
    c1 = ((c1 - 0x70) * 2) - 1;
    c2 = c2 - 0x1f;
  }

  // JISx0208変換の表示
  // ----- JISxからフォントのアドレスを求める
  // JISxの区点を求める
  uint32_t MSB = c1 - 0x20;//区
  uint32_t LSB = c2 - 0x20;//点
  // JISの句点番号で分類
  uint32_t Address = 0;
  // 各種記号・英数字・かな(一部機種依存につき注意,㍍などWindowsと互換性なし)
  if (MSB >= 1 && MSB <= 15 && LSB >= 1 && LSB <= 94) {
    Address = ( (MSB - 1) * 94 + (LSB - 1)) * 32;
  }
  /*第一水準*/
  if (MSB >= 16 && MSB <= 47 && LSB >= 1 && LSB <= 94) {
    Address = ( (MSB - 16) * 94 + (LSB - 1)) * 32 + 43584;
  }
  /*第二水準*/
  if (MSB >= 48 && MSB <= 84 && LSB >= 1 && LSB <= 94) {
    Address = ((MSB - 48) * 94 + (LSB - 1)) * 32 + 138464;
  }

  // GT20L16J1Y内部では1区と同等の内容が収録されている
  if (MSB == 85 && LSB >= 0x01 && LSB <= 94) {
    Address = ((MSB - 85) * 94 + (LSB - 1)) * 32 + 246944;
  }

  // GT20L16J1Y内部では2区、3区と同等の内容が収録されている
  if (MSB >= 88 && MSB <= 89 && LSB >= 1 && LSB <= 94) {
    Address = ((MSB - 88) * 94 + (LSB - 1)) * 32 + 249952;
  }

  /*漢字ROMにデータを送信*/
  digitalWrite(SS, HIGH);
  digitalWrite(SS, LOW);  //通信開始
  SPI.transfer(0x03);
  SPI.transfer(Address >> 16  & 0xff);
  SPI.transfer(Address >> 8   & 0xff);
  SPI.transfer(Address        & 0xff);
  /*漢字ROMからデータを受信*/
  for (int i = 0; i < 32; i++) {
    matrixData[i] = SPI.transfer(0x00);
  }
  digitalWrite(SS, HIGH); //通信終了
}

// 漢字ROM READ : ASCIIコード
void KANJIROM::readASCIIFont(uint8_t asciiCode)
{
  uint32_t Address = 0;
  // ASCII文字
  if (asciiCode >= 0x20 && asciiCode <= 0x7F) {
    Address = ( asciiCode - 0x20) * 16 + 255968;
  }

  // 漢字ROMにデータを送信
  digitalWrite(SS, HIGH);
  digitalWrite(SS, LOW);  //通信開始
  SPI.transfer(0x03);
  SPI.transfer(Address >> 16  & 0xff);
  SPI.transfer(Address >> 8   & 0xff);
  SPI.transfer(Address        & 0xff);
  // 漢字ROMからデータを受信
  for (int i = 0; i < 16; i++) {
    matrixData[i] = SPI.transfer(0x00);
  }
  digitalWrite(SS, HIGH); //通信終了
}
