#include <Arduino.h>
#include "u2s.h"
#include "u2sTable.h"

// UTF16->SJIS変換
uint16_t U2S::UTF16ToSJIS(uint16_t utf16) {
  uint16_t sjisCode = 0x00 ;
  uint8_t codeH = utf16 >> 8 ;
  uint8_t codeL = utf16 & 0x00ff ;

  uint8_t *mapTable = NULL ;
  switch (codeH) {
    case 0x00: mapTable = (uint8_t *)code00 ; break ;
    case 0x03: mapTable = (uint8_t *)code03 ; break ;
    case 0x04: mapTable = (uint8_t *)code04 ; break ;
    case 0x20: mapTable = (uint8_t *)code20 ; break ;
    case 0x21: mapTable = (uint8_t *)code21 ; break ;
    case 0x22: mapTable = (uint8_t *)code22 ; break ;
    case 0x23: mapTable = (uint8_t *)code23 ; break ;
    case 0x25: mapTable = (uint8_t *)code25 ; break ;
    case 0x26: mapTable = (uint8_t *)code26 ; break ;
    case 0x30: mapTable = (uint8_t *)code30 ; break ;
    case 0x4E: mapTable = (uint8_t *)code4E ; break ;
    case 0x4F: mapTable = (uint8_t *)code4F ; break ;
    case 0x50: mapTable = (uint8_t *)code50 ; break ;
    case 0x51: mapTable = (uint8_t *)code51 ; break ;
    case 0x52: mapTable = (uint8_t *)code52 ; break ;
    case 0x53: mapTable = (uint8_t *)code53 ; break ;
    case 0x54: mapTable = (uint8_t *)code54 ; break ;
    case 0x55: mapTable = (uint8_t *)code55 ; break ;
    case 0x56: mapTable = (uint8_t *)code56 ; break ;
    case 0x57: mapTable = (uint8_t *)code57 ; break ;
    case 0x58: mapTable = (uint8_t *)code58 ; break ;
    case 0x59: mapTable = (uint8_t *)code59 ; break ;
    case 0x5A: mapTable = (uint8_t *)code5A ; break ;
    case 0x5B: mapTable = (uint8_t *)code5B ; break ;
    case 0x5C: mapTable = (uint8_t *)code5C ; break ;
    case 0x5D: mapTable = (uint8_t *)code5D ; break ;
    case 0x5E: mapTable = (uint8_t *)code5E ; break ;
    case 0x5F: mapTable = (uint8_t *)code5F ; break ;
    case 0x60: mapTable = (uint8_t *)code60 ; break ;
    case 0x61: mapTable = (uint8_t *)code61 ; break ;
    case 0x62: mapTable = (uint8_t *)code62 ; break ;
    case 0x63: mapTable = (uint8_t *)code63 ; break ;
    case 0x64: mapTable = (uint8_t *)code64 ; break ;
    case 0x65: mapTable = (uint8_t *)code65 ; break ;
    case 0x66: mapTable = (uint8_t *)code66 ; break ;
    case 0x67: mapTable = (uint8_t *)code67 ; break ;
    case 0x68: mapTable = (uint8_t *)code68 ; break ;
    case 0x69: mapTable = (uint8_t *)code69 ; break ;
    case 0x6A: mapTable = (uint8_t *)code6A ; break ;
    case 0x6B: mapTable = (uint8_t *)code6B ; break ;
    case 0x6C: mapTable = (uint8_t *)code6C ; break ;
    case 0x6D: mapTable = (uint8_t *)code6D ; break ;
    case 0x6E: mapTable = (uint8_t *)code6E ; break ;
    case 0x6F: mapTable = (uint8_t *)code6F ; break ;
    case 0x70: mapTable = (uint8_t *)code70 ; break ;
    case 0x71: mapTable = (uint8_t *)code71 ; break ;
    case 0x72: mapTable = (uint8_t *)code72 ; break ;
    case 0x73: mapTable = (uint8_t *)code73 ; break ;
    case 0x74: mapTable = (uint8_t *)code74 ; break ;
    case 0x75: mapTable = (uint8_t *)code75 ; break ;
    case 0x76: mapTable = (uint8_t *)code76 ; break ;
    case 0x77: mapTable = (uint8_t *)code77 ; break ;
    case 0x78: mapTable = (uint8_t *)code78 ; break ;
    case 0x79: mapTable = (uint8_t *)code79 ; break ;
    case 0x7A: mapTable = (uint8_t *)code7A ; break ;
    case 0x7B: mapTable = (uint8_t *)code7B ; break ;
    case 0x7C: mapTable = (uint8_t *)code7C ; break ;
    case 0x7D: mapTable = (uint8_t *)code7D ; break ;
    case 0x7E: mapTable = (uint8_t *)code7E ; break ;
    case 0x7F: mapTable = (uint8_t *)code7F ; break ;
    case 0x80: mapTable = (uint8_t *)code80 ; break ;
    case 0x81: mapTable = (uint8_t *)code81 ; break ;
    case 0x82: mapTable = (uint8_t *)code82 ; break ;
    case 0x83: mapTable = (uint8_t *)code83 ; break ;
    case 0x84: mapTable = (uint8_t *)code84 ; break ;
    case 0x85: mapTable = (uint8_t *)code85 ; break ;
    case 0x86: mapTable = (uint8_t *)code86 ; break ;
    case 0x87: mapTable = (uint8_t *)code87 ; break ;
    case 0x88: mapTable = (uint8_t *)code88 ; break ;
    case 0x89: mapTable = (uint8_t *)code89 ; break ;
    case 0x8A: mapTable = (uint8_t *)code8A ; break ;
    case 0x8B: mapTable = (uint8_t *)code8B ; break ;
    case 0x8C: mapTable = (uint8_t *)code8C ; break ;
    case 0x8D: mapTable = (uint8_t *)code8D ; break ;
    case 0x8E: mapTable = (uint8_t *)code8E ; break ;
    case 0x8F: mapTable = (uint8_t *)code8F ; break ;
    case 0x90: mapTable = (uint8_t *)code90 ; break ;
    case 0x91: mapTable = (uint8_t *)code91 ; break ;
    case 0x92: mapTable = (uint8_t *)code92 ; break ;
    case 0x93: mapTable = (uint8_t *)code93 ; break ;
    case 0x94: mapTable = (uint8_t *)code94 ; break ;
    case 0x95: mapTable = (uint8_t *)code95 ; break ;
    case 0x96: mapTable = (uint8_t *)code96 ; break ;
    case 0x97: mapTable = (uint8_t *)code97 ; break ;
    case 0x98: mapTable = (uint8_t *)code98 ; break ;
    case 0x99: mapTable = (uint8_t *)code99 ; break ;
    case 0x9A: mapTable = (uint8_t *)code9A ; break ;
    case 0x9B: mapTable = (uint8_t *)code9B ; break ;
    case 0x9C: mapTable = (uint8_t *)code9C ; break ;
    case 0x9D: mapTable = (uint8_t *)code9D ; break ;
    case 0x9E: mapTable = (uint8_t *)code9E ; break ;
    case 0x9F: mapTable = (uint8_t *)code9F ; break ;
    case 0xFF: mapTable = (uint8_t *)codeFF ; break ;
  }

  if (mapTable == NULL) {
    return 0x0000 ;
  }

  for (int i=0;i<256;i++) {
    int readPos = i * 3 ;
    uint8_t utf = pgm_read_byte(mapTable + readPos) ;
    if (utf == codeL) {
      uint8_t sjis1 = pgm_read_byte(mapTable + readPos + 1) ;
      uint8_t sjis2 = pgm_read_byte(mapTable + readPos + 2) ;
      sjisCode = sjis2 << 8 | sjis1 ;
      break ;
    } else
    if (utf > codeL) {
      break ;
    } else
    if (utf == 0xFF) {
      uint8_t sjis1 = pgm_read_byte(mapTable + readPos + 1) ;
      uint8_t sjis2 = pgm_read_byte(mapTable + readPos + 2) ;
      sjisCode = sjis2 << 8 | sjis1 ;
      break ;
    }
  }
  return sjisCode ;
}

// UTF8->UTF16
uint16_t U2S::UTF8ToUTF16(uint8_t *codes) {
  uint16_t utf16Code = 0x0000 ;
  int codeLen = UTF8LEN(codes) ;
 
  if (codeLen == 2) {
    ((uint8_t *)&utf16Code)[0] = ((codes[0] & 0x1F)<<6) + (codes[1] & 0x3F);
    ((uint8_t *)&utf16Code)[1] = ((codes[0] & 0x1F)>>2);
  } else
  if (codeLen == 3) {
    ((uint8_t *)&utf16Code)[1] = ((codes[0] & 0xF)<<4) + ((codes[1] & 0x3F)>>2);
    ((uint8_t *)&utf16Code)[0] = ((codes[1] & 0x3F)<<6) + (codes[2] & 0x3F);
  }
  return utf16Code ;
}

// UTF8の文字長(2 or 3)を返す
// ASCIIの場合は1を返す
int U2S::UTF8LEN(uint8_t *codes) {
  int utfLen = 0 ;
  if( codes[0] < 0x80 ){
    utfLen = 1 ;
  } else
  if( codes[0] < 0xE0 ){
    utfLen = 2 ;
  } else {
    utfLen = 3 ;
  }
  return utfLen ;
}
